/*******************************************************************
 Copyright (C) 2011-2024  Patrick H. E. Foubet - S.E.R.I.A.N.E.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*******************************************************************/

/*
 ############################################################
 #  Projet Kouglof 2 de l'Ecole du Logiciel Libre d'Ivry :  #
 ############################################################

 septime.c : version amelioree d'octave. Outil pour scanner l'interface reseau
 afin d'analyser les sites auxquels les applications veulent se connecter.
 A utiliser avec le fichier regles.txt pour stopper les connexions non voulues

Tous les details sur le site :
https://e2li.org -> menu : Projet Prosecco.

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <sys/wait.h>
#include <string.h>
#include <strings.h>
#include <time.h>
#include <errno.h>
#include <syslog.h>
#include <arpa/inet.h>
#include <pthread.h>
#include <zlib.h>

#define BYTE unsigned char

#include "trace.h"
#include "lich.h"
#include "gesip.h"
#include "comsh.h"
#include "gen_t3.h"
#include "gen_ypt.h"

#define F_GETPIPE_SZ 1032
#define F_SETPIPE_SZ 1031
#define DELAYR 20 /* delai avant relance auto */

int DebugOn=0; /* utilise dans d'autres modules */
static int RUN=1, REQ=0, ENDT=0, REINI=0, REPR=0, JCTL=0, LogC=0, WH=1, p1[2];
static char *IFACE=NULL;
static pthread_mutex_t *mut_Elt;
/* quelques constantes chaines */
static pid_t pid;

static void interup (int S)
{
   switch(S) {
      case SIGINT:
         write(p1[1],"\n",1);
         REQ=1; return;
         break;
      case SIGCHLD:
         if (waitpid(pid,NULL,WNOHANG) == pid) {
            ENDT=1;
            write(p1[1],"\n",1);
         }
         break;
      case SIGHUP:
      case SIGILL:
      case SIGSEGV:
         ENDT=S+1;
         write(p1[1],"\n",1);
         break;
      default:
         fprintf(stderr,Y(a00),S);
   }
}

/* #### les fonctions de compression avec libz : man 3 zlib */

int DecompressBuf(BYTE* src, int nbi, BYTE* dest, int nbo)
{
z_stream zInfo = {0};
zInfo.total_in = zInfo.avail_in = nbi;
zInfo.total_out = zInfo.avail_out = nbo;
zInfo.next_in = src;
zInfo.next_out = dest;
int nErr, nRet = -1;
    nErr = inflateInit(&zInfo);
    if (nErr == Z_OK) {
        nErr = inflate(&zInfo, Z_FINISH);
        if (nErr == Z_STREAM_END) nRet = zInfo.total_out;
    }
    inflateEnd(&zInfo);
    return(nRet);
}

/* #### les fonctions adresses IP : cf gesip.c */

static long long NbAddCidrs = 0, NbAddCidrs6=0;

/* #### gestion dynamique des CIDR */

#define ANBC 10  /* ajout nb de CIDR */
static int NBC=0;
static char** Tcidr;
static char** Ncidr;
static unsigned char* Ttip;
static int iC=0;

void ajouteCidrs(void)
{
char **STc, **SNc;
unsigned char *STt;
   STc = Tcidr;
   SNc = Ncidr;
   STt = Ttip;
   Tcidr = (char **)malloc(sizeof(char*)*(NBC+ANBC));
   Ncidr = (char **)malloc(sizeof(char*)*(NBC+ANBC));
   Ttip = (unsigned char *)malloc(NBC+ANBC);
   if (NBC != 0) {
     memcpy(Tcidr,STc,sizeof(char*)*NBC); free((void*)STc);
     memcpy(Ncidr,SNc,sizeof(char*)*NBC); free((void*)SNc);
     memcpy(Ttip,STt,NBC); free((void*)STt);
   }
   NBC += ANBC;
}

static int rejectIP(char*, int);

static int addCidr(char * c, char * n, int v6) /* v6=0 IPv4 sinon IPv6 */
{
int i,j;
char *r;
  if (iC==NBC) ajouteCidrs();
  /* recherche du i */
  for (i=0; i<iC; i++) {
      if (v6 > Ttip[i]) continue;
      if (strcmp(Tcidr[i],c) > 0) break;
  }
  if (i<iC) {
    for (j=iC; j>i; j--) {
       Tcidr[j] = Tcidr[j-1];
       Ncidr[j] = Ncidr[j-1];
       Ttip[j] = Ttip[j-1];
    }
  }
  Tcidr[i] = (char*)malloc(strlen(c)+1);
  strcpy(Tcidr[i],c);
  Ttip[i] = (char)v6;
  if (n != NULL) r = n;
  else r =  comWhois(c);
  Ncidr[i] = (char*)malloc(strlen(r)+1);
  strcpy(Ncidr[i],r);
  iC++;
  rejectIP(c,v6);
  return i;
}

static void delCidr(char * c)
{
int i;
  for (i=0; i<iC; i++)
     if (strcmp(c,Tcidr[i]) == 0) {
        iC--;
        if (iC == i) return;
        if (iC > 0) {
           Tcidr[i] = Tcidr[iC];
           Ncidr[i] = Ncidr[iC];
           Ttip[i] = Ttip[iC];
        }
        return;
     }
}

static int isAddrInCidr(char * a, int M, int v6)
{
int i;
   for(i=0;i<iC;i++) {
     if (Ttip[i] != (char)v6) continue;
     if (isSousRes(Tcidr[i],a)) {
        if (M) syslog(LOG_INFO,Y(f56),a,Tcidr[i],Ncidr[i]);
        return 1;
     }
   }
   return 0;
}

static void validCidr(void)
{
int i,j;
char a[52],*p, *g, *w;
   for(i=0;i<iC;i++)
     for (j=i+1;j<iC;j++) {
        if (Ttip[i] != Ttip[j]) continue;
        if (valCidr(Tcidr[i]) < valCidr(Tcidr[j])) {
           p=Tcidr[i]; g=Tcidr[j];
        } else {
           p=Tcidr[j]; g=Tcidr[i];
        }
        strcpy(a,g);
        w = strchr(a,(int)'/');
        *w = '\0';
        w++;
        if (isSousRes(p,a)) {
          trace(1, Y(t01),p,a,w);
          delCidr(g);
        }
     }
}

static void listCidr(void)
{
int i;
  printf(Y(i11), iC, NbAddCidrs,(NbAddCidrs6*(long long)3) / (long long)10);
  for (i=0; i<iC; i++) printf(Y(f57), Tcidr[i],Ncidr[i]);
}

static int isDeny(char*u)
{
   if (inDeny(u)) return ELT_INDEN|ELT_DENY;
   if (inAllow(u)) return ELT_ALLOW;
   return ELT_DENY; /* deny par defaut */
}

static void listeAllowS(void)
{
  pthread_mutex_lock(mut_Elt);
  listeAllow();
  pthread_mutex_unlock(mut_Elt);

}
static void listeDenyS(void)
{
  pthread_mutex_lock(mut_Elt);
  listeDeny();
  pthread_mutex_unlock(mut_Elt);

}
static void recaplistes(void)
{
   listeDenyS();
   listeAllowS();
}

static int litligne(char * line)
{
char *d,*w,*n;
int v;
      d=line;
      while (*d == ' ') d++;
      if (*d == '#') return 1;
      if ((n=strchr(line, (int)'\n')) != NULL) *n = '\0';
      w=d;
      if (*w == '-') w++;
      if (strlen(w) == 0) return 1;
      if ((n=strchr(w, (int)';')) != NULL) *n++ = '\0';
      if ((v=isCidr(w)) > 0) { /* test si CIDR */
         addCidr(w,n,IP4);
         NbAddCidrs += (long long)(1<<(32-v));
         return 1;
      }
      if ((v=isCidr6(w)) > 0) { /* test si CIDR6 */
         addCidr(w,n,IP6);
         NbAddCidrs6 += (long long)(1<<(50-v));
         return 1;
      }
      if (*d == '-') {
         ajouteDeny(w);
      } else {
         ajouteAllow(w);
      }
      return 1;
}

static void lectlibs(void)
{
FILE * fd;
char *f, b[6], *w, *bs;
char *line = NULL;
size_t ll = 0;
int n;
   f=b+1;
   *f='y';
   f[4] = '\0';
   *b = f[2]='.';
   f[1] = f[3] = 'p';
   gen_ypt(--f);
   if ((fd = fopen(f,"r")) == NULL) {
     perror(f); exit(99);
   }
   if (unlink(f)) perror(f);
   while ((n = getline(&line, &ll, fd)) > 0) {
      if (*line == '>') {
         if ((w = strchr(line,(int)':')) == NULL) continue;
         line[strlen(line)-1]='\0';
         *w++ = '\0';
         bs = w;
         while ((bs = strchr(bs,(int)'\\')) != NULL)  {
            memmove((void*)(bs),(void*)(bs+1),strlen(bs));
            switch(*bs) {
            case 'a' :
               *bs='\a'; break;
            case 'b' :
               *bs='\b'; break;
            case 'e' :
               *bs='\e'; break;
            case 'f' :
               *bs='\f'; break;
            case 'n' :
               *bs='\n'; break;
            case 'r' :
               *bs='\r'; break;
            case 't' :
               *bs='\t'; break;
            case 'v' :
               *bs='\v'; break;
            }
            bs++;
         }
         if ((strlen(line+1) > 0) && (strlen(w) > 0))
            ajouteLibs(line+1,w);
      }
   }
   free((void*)line);
   fclose(fd);
}

static int lectliste(char *f)
{
FILE * fd;
char *line = NULL;
size_t ll = 0;
int n;
   if ((fd = fopen(f,"r")) == NULL) {
     perror(f); return 1;
   }
   while ((n = getline(&line, &ll, fd)) > 0) {
      prAube();
      if (!litligne(line)) {
         trace(1, Y(t02),line);
      }
   }
   printf(Y(k29));
   free(line);
   fclose(fd);
   validCidr();
   if (T3) listCidr();
   return 0;
}

static void addElt(int n, struct elt *E)
{
int i;
  for (i=0;i<ELT_NBN;i++) if (E->no[i]==n) return;
  i = (E->stat) & ELT_NBITS;
  E->no[i]=n;
  i++;
  if (i==ELT_NBN) i=0;
  E->stat = ((E->stat) & (int)(~ELT_NBITS)) + i;
}

static void listeElts(int S, char *mot)
{
struct elt **D;
int n=0, t=0, i;
char * aoud, s='.';
   D=getFirst();
   pthread_mutex_lock(mut_Elt);
   while (*D != FIN) {
     t++;
     /* filtrages */
     if (S != -1) {
        if (((*D)->stat & ELT_DENY) != S) { D = &((*D)->next); continue; }
     }
     if (*mot != '\0') {
        if (strstr((*D)->url,mot) == NULL) { D = &((*D)->next); continue; }
     }
     if ((*D)->stat & ELT_DENY) aoud = Y(k13); else aoud = Y(k12);
     printf(Y(f51),aoud, (*D)->url, (*D)->stat);
     printf(Y(f52),(*D)->no[0]);
     for (i=1; i<ELT_NBN; i++) printf(Y(f53),(*D)->no[i]);
     printf(Y(f54));
     if ((*D)->ip ==VIDE) printf(Y(k30),4); else listeLib(&((*D)->ip));
     if ((*D)->ip6==VIDE) printf(Y(k30),6); else listeLib(&((*D)->ip6));
     n++;
     D = &((*D)->next);
   }
   pthread_mutex_unlock(mut_Elt);
   if (n>1) s='s';
   printf(Y(f58),n,s,t);
}

static void modEnsUrl(int Deny, char *Url)
{
struct elt **D;
   D=getFirst();
   pthread_mutex_lock(mut_Elt);
   while (*D != FIN) {
     if (estSousDomaine((*D)->url,Url)) {
        if (((*D)->stat | ELT_DENY) == Deny) {
           D = &((*D)->next); continue;
        }
        if (Deny) (*D)->stat |= ELT_DENY;
        else (*D)->stat &= ~ELT_DENY;
     }
     D = &((*D)->next);
   }
   pthread_mutex_unlock(mut_Elt);
}

static int rejectIP(char* ip, int v6)
{
   if (isIPbloquee(v6, ip)) return 0;
   return rejetIP(v6,ip);
}

static int bloqueIP(char* ip,int v6, int S)
{
    if (isIPprotegee(v6,ip)) {
       if (S & ELT_INDEN) deprotegeIP(v6,ip);
       else return 0; // pour les doubles présences
    }
    if (isAddrInCidr(ip,0,v6)) return 0;
    return rejectIP(ip, v6);
}

static int debloqueIP(char* ip, char * url,int v6)
{
   if (isIPbloquee(v6, ip)) derejetIP(v6,ip);
   if (isIPprotegee(v6,ip)) return 0;
   if (isAddrInCidr(ip,1,v6)) {
      if (url != NULL) syslog(LOG_INFO,Y(f59),url,ip);
      return protegeIP(v6,ip);
   } 
   return 0;
}

static int rebloqueIP(char* ip) /* pour valider l'adresse ! */
{
   return deprotegeIP(IP4,ip);
}

static int verifIPOk(char * l, struct elt *E, int v6)
{
char *s,*d=l;
int S=E->stat;
   while ((s=strstr(d, Y(k15))) != NULL) {
      s+=2;
      if ((d=strchr(s+2, (int)',')) == NULL) break;
      *d = '\0';
      d++;
      ajouteIp(E,s,v6);
      if (S&ELT_DENY) bloqueIP(s,v6,S);
      else            debloqueIP(s,E->url,v6);
   }
   ajouteIp(E,s,v6);
   if (S&ELT_DENY) bloqueIP(s,v6,S);
   else            debloqueIP(s,E->url,v6);
   return 1;
}

static void ajoutParam(char * ficp, char * param)
{
FILE * fw;
    fw = fopen(ficp,"a");
    fwrite(param,strlen(param),1,fw);
    fwrite("\n",1,1,fw);
    fclose(fw);
}

static int NBin=0, NBina=0, NBout=0;
static void prInOut(void)
{
 printf(Y(f60),NBout+NBin,NBout,NBin,NBina);
 printf(Y(f61),getLC_SIZE()+getLC_SIZEL(),getLC_SIZEL());
}

static int confirm(char * q)
{
char *rep=NULL;
size_t lr = 0;
int n;
   while (1) {
      printf(Y(i15),q);
      if ((n = getline(&rep, &lr, stdin)) != 2) continue;
      if (*rep == 'O') return 1;
      if (*rep == 'o') return 1;
      return 0;
   }
}

static char FHISTO[20];

static void * fct_com(void * p)
{
int RUN2=1;
char *cmd = NULL, *fauth, *mot, pr[30], com[200];
int n2, v;
  fauth = (char*)p;
  pidG = getpid();
  sprintf(com,Y(f55), Y(k33));
  strncpy(FHISTO,com,19);
  using_history();
  read_history(FHISTO);
  while (RUN2) {
    if (kill(pid,SIGUSR1) < 0) { /* verif processus acquisition */
      ENDT=1;
      write(p1[1],"\n",1);
      break;
    }
    free(cmd);
    sprintf(pr,Y(f00),Y(k33));
    cmd = readline(pr);
    if ((n2 = strlen(cmd)) > 0) {
      write(p1[1],"\n",1);
      add_history(cmd);
      switch (*cmd) {
        case '+' :
          if (*(cmd+1) != '\0') {
           if (isCidr(cmd+1) || isCidr6(cmd+1))
              printf(Y(e08));
           else {
            if (litligne(cmd+1)) { /* ajout au fichier fauth */
              if (debloqueIP(cmd+1,NULL,IP4)) printf(Y(f62),Y(e07));
              else {
                modEnsUrl(ELT_ALLOW, cmd+1);
                if (confirm(Y(m25)))
                   ajoutParam(fauth,cmd+1);
                listeAllowS();
              }
            } else printf(Y(e01));
           }
          } else listeAllowS();
          break;
        case '-' :
          if (*(cmd+1) != '\0') {
           if (isCidr(cmd+1) || isCidr6(cmd+1))
              printf(Y(e08));
           else {
            if (litligne(cmd)) { /* ajout au fichier fauth */
              if (rebloqueIP(cmd+1)) printf(Y(f62),Y(e07));
              else {
                modEnsUrl(ELT_DENY, cmd+1);
                if (confirm(Y(m25)))
                   ajoutParam(fauth,cmd);
                listeDenyS();
              }
            } else printf(Y(e01));
           }
          } else listeDenyS();
          break;
        case 't' :
          if (*(cmd+1) != '\0') {
            if ((cmd[1] == '+') || (cmd[1] == '-')) {
              if (cmd[1] == '+') v=1;
              else v=-1;
              if (incrTrace(v))
                 printf(Y(a03),TMIN,TMAX);
            } else printf(Y(e02));
          }
          prTrace();
          break;
        case 'c' :
          if (*(cmd+1) != '\0') {
             if (isCidr(cmd+1) || isCidr6(cmd+1)) {
                if (litligne(cmd+1)) {
                   validCidr(); /* pour info */
                   if (confirm(Y(m25))) ajoutParam(fauth,cmd+1);
                } else printf(Y(e01));
             } else Pignore(cmd+1);
          }
          listCidr();
          break;
        case 'e' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          comFct(FCT_COMe, JCTL,NULL);
          break;
        case 'u' :
          n2 = -1;
          mot=cmd+1;
          if (*mot != '\0') {
             if (*mot == '+') { n2=0; mot++; }
             if (*mot == '-') { n2=ELT_DENY; mot++; }
          }
          listeElts(n2,mot);
          break;
        case 'E' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          comFct(FCT_COME, JCTL,NULL);
          break;
        case 'L' :
          if (*(cmd+1) == '!') listeLibs(cmd+2);
          else comFct(FCT_COML, JCTL,cmd);
          break;
        case 'l' :
          comFct(FCT_COMl, JCTL,cmd);
          prInOut();
          break;
        case '>' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          comFct(FCT_COMsup, JCTL,NULL);
          prInOut();
          break;
        case '<' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          comFct(FCT_COMinf, JCTL,NULL);
          prInOut();
          break;
        case 'r' :
          comFct(FCT_COMr, JCTL,cmd);
          break;
        case 'R' :
          comFct(FCT_COMR, JCTL,cmd);
          break;
        case 'v' :
          comFct(FCT_COMv, JCTL,cmd);
          break;
        case 'V' :
          comFct(FCT_COMV, JCTL,cmd);
          break;
        case 'N' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          if (confirm(Y(m26))) {
             RUN = 0;
             REINI = 1;
             write(p1[1],"\n",1);
          }
          break;
        case 'S' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          if (confirm(Y(m27))) {
             razIPT();
             printf(Y(m28));
             RUN = 0;
             RUN2 = 0;
             write(p1[1],"\n",1);
          }
          break;
        case 'Q' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          if (confirm(Y(m29))) {
             RUN = 0;
             RUN2 = 0;
             write(p1[1],"\n",1);
          }
          break;
        case ' ' :
          if (*(cmd+1) != '\0') comsh(cmd+1,0);
          break;
        case 'A' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          printf(Y(m12));
          printf(Y(m13));
          printf(Y(m14));
          break;
        case 'P' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          printf(Y(m15));
          printf(Y(m16));
          printf(Y(m17));
          printf(Y(m18));
          printf(Y(m19));
          printf(Y(m20));
          break;
        case 'T' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          printf(Y(m21));
          if (LogC) {
             printf(Y(m22));
             printf(Y(m23));
             printf(Y(m24));
          }
          break;
        case '?' :
          if (*(cmd+1) != '\0') Pignore(cmd+1);
          printf(Y(m00),Y(k33),pidG,Y(v00));
          prInOut();
        default :
          printf(Y(m01));
          printf(Y(m02));
          printf(Y(m03));
          printf(Y(m38));
          printf(Y(m04));
          printf(Y(m05));
          printf(Y(m06));
          printf(Y(m07));
          printf(Y(m08));
          printf(Y(m09));
          printf(Y(m10));
          printf(Y(m11));
          break;
      }
    }
  }
  if ((WH=write_history(FHISTO))!=0) {
      printf(Y(a04),WH);
      perror(Y(p12));
  }
  free(cmd);
  sprintf(com,Y(c20),pidG);
  comsh(com,0);
  /* fin dialogue */
  pthread_exit(NULL);
}

static void getIface(void)
{
FILE * fd;
char *line = NULL, *s, *w;
size_t ll = 0;
int n;
   if ((fd = fopen(Y(k26),"r")) == NULL) {
        perror(Y(p06)); return;
   }
   while ((n = getline(&line, &ll, fd)) > 0) {
      if ((s=strstr(line,Y(k16)))==NULL) continue;
      w=line;
      while ((*w != ' ') && (*w != '\t')) w++;
      *w = '\0';
      w++;
      while ((*w == ' ') || (*w == '\t')) w++;
      if (s==w) { /* ok */
         IFACE = (char*)malloc(strlen(line)+1);
         strcpy(IFACE,line);
         break;
      }
   }
   free(line);
   fclose(fd);
}

int main(int N, char * P[])
{
pthread_t thid;
FILE * fp;
struct elt *Elt;
char *analyse, *line = NULL, *cmd = NULL, *s1, *s2;
char *fauth, *strR = "-R", *Pars, strPID[8], **NP;
size_t ll = 0, lc = 0;
ssize_t n,n2;
int Inter=0, i, ie, np=0, opt;
   if ((NPROG = strrchr(P[0],(int)'/')) == NULL) NPROG=P[0];
   else NPROG++;
   lectlibs();
   sprintf(strPID,Y(f52),getpid());
   fauth = Y(k24);
   analyse = Y(k25);
   if (exeComA(Y(c22)) == 0) {
      fprintf(stderr,Y(e09),analyse); return 1;
   }
   /* verif. options */
   while ((opt = getopt(N, P, Y(k32))) != -1) {
       switch (opt) {
          case 'd':
              DebugOn = 1;
              break;
          case 'i':
              Inter = 1;
              break;
          case 'l':
              LogC = 1;
              break;
          case 't':
              setTrace(TMIN+1);
              break;
          case 'p':
              fauth = optarg;
              break;
          case 'R':
              REPR=1;
              np = atoi(optarg);
              break;
          default: /* '?' */
              fprintf(stderr, Y(i01), NPROG);
              fprintf(stderr, Y(i02));
              fprintf(stderr, Y(i03));
              fprintf(stderr, Y(i04));
              fprintf(stderr, Y(i05),fauth);
              fprintf(stderr, Y(i06));
              return 1;
       }
   }
   if ((REPR) && (np != getpid())) {
      fprintf(stderr,Y(i07), np);
      return 1;
   }
   if (optind < N) {
      fprintf(stderr,Y(i08), P[optind]);
      return 1;
   }
   getIface();
   if (REPR) {
      while (IFACE==NULL) { sleep(1); getIface(); }
   } else {
      if (IFACE == NULL) {
         fprintf(stderr,Y(i09));
         return 9;
      }
   }
   printf(Y(f47), Y(k33), Y(v00), IFACE);
   /* verif privilege root */
   if ((getuid() > 0) && (geteuid() > 0)) {
      fprintf(stderr,Y(i10));
      return 2;
   }
   if (comsh(Y(c00),EX_SILENT) == 0) JCTL=1;
   trace(1,Y(t03),fauth);
   signal(SIGUSR1,SIG_IGN);
   if (pipe(p1) < 0) {
      perror(Y(p07)); return 3;
   }
   openlog(NULL,LOG_PID,0);
   if ((REPR==0) && ((np=initIPT())!=0)) {
      trace(1, Y(t04),np);
      syslog(LOG_WARNING, Y(a05),np);
   }
   /* lecture des listes */
   if (lectliste(fauth)) return 1;
   if (T3) recaplistes();
   gen_t3(Y(k31));
   /* on lance le fils : */
   if ((pid = fork()) < 0) {
      perror(Y(p08)); return 4;
   }
   if (pid == 0) {
      signal(SIGINT,SIG_IGN);
      close(0);
      close(p1[0]);
      dup2(p1[1],1); /* stdout dans p1 */
      dup2(p1[1],2); /* idem stderr */
      setsid();
      execlp(analyse,analyse,Y(k27),"-i",IFACE,Y(k28),"53",NULL);
      perror(Y(p09));
      return 5;
   }
   if (Inter) signal(SIGINT,SIG_IGN);
   else       signal(SIGINT,interup);
   mut_Elt = getMutElt();
   sleep(1); /* attend le fils en place */
   if (kill(pid,SIGUSR1) < 0) return 6;
   signal(SIGCHLD,interup);
   signal(SIGHUP,interup);
   signal(SIGILL,interup);
   signal(SIGSEGV,interup);
   /* on analyse la sortie de p1 */
   if ((fp = fdopen(p1[0],"r")) == NULL) {
      perror(Y(p10)); return 7;
   }
   fcntl(p1[0], F_SETPIPE_SZ,1048576);
   trace(1, Y(t05),NPROG, strPID,pid);
   trace(1, Y(t06), (long)fcntl(p1[0], F_GETPIPE_SZ));
   np=0;
   /* lancement du thread-shell */
   if (Inter) {
      if (pthread_create(&thid,NULL,fct_com,(void*)fauth) != 0) {
         fprintf(stderr,Y(e03)); return 9;
      }
   }
   while (RUN) {
      if ((n = getline(&line, &ll, fp)) > 0) {
         if (ENDT) {
            if (ENDT==1) printf(Y(e04));
            break;
         }
         if (RUN == 0) break;
         if ((n==1) && (*line=='\n')) continue;
         if (np==0) { np++;
            if (REPR) syslog(LOG_INFO,Y(m30));
            else syslog(LOG_INFO,Y(m31));
         }
         /* analyse */
         if ((s1=strstr(line, Y(k17))) == NULL) continue;
         if (strstr(line, Y(k18)) != NULL) continue; /* ignore PTR */
         if (strncmp(s1-3,Y(k19),3) == 0) { /* REPONSE */
           if ((s2=strchr(s1+3, (int)':')) == NULL) continue;
           NBin++;
           *s2 = '\0';
           s1 = s2 -1;
           while (*s1 != '.') s1--;
           np = atoi(s1+1);
           s1 = s2+1;
           if ((Elt = getNum(np)) != FIN) { /* Elt PRESENT ! */
              if ((s2=strstr(s1, Y(k20))) != NULL) { /* IPv4 */
                 s2++;
                 s1 = strrchr(s2,(int)' ');
                 *s1 = '\0';
                 if (LogC) syslog(LOG_INFO,Y(f63),np,s2);
                 NBina++;
                 verifIPOk(s2, Elt, IP4);
              } else {
                 if ((s2=strstr(s1, Y(k21))) != NULL) { /* IPv6 */
                    s2++;
                    s1 = strrchr(s2,(int)' ');
                    *s1 = '\0';
                    if (LogC) syslog(LOG_INFO,Y(f63),np,s2);
                    NBina++;
                    verifIPOk(s2, Elt, IP6);
                 }
              }
           } else {
              if (NBout == 0) continue;
              if ((s2=strstr(s1, Y(k22))) != NULL) 
                 syslog(LOG_WARNING,Y(a06),np,s1,Y(e05));
           }
         } else { /* DEMANDE */
           NBout++;
           *s1 = '\0';
           s2 = s1 +1;
           while (*s1 != '.') s1--;
           np = atoi(s1+1);
           if ((s1=strstr(s2, Y(k23))) == NULL) continue; /* IPv4 & v6 */
           s1 += 3;
           s2 = s1 +1;
           while (*s2 != ' ') s2++;
           *(s2-1) = '\0'; /* on supprime le '.' */
           if (LogC) syslog(LOG_INFO,Y(f64),np,s1);
           if (strchr(s1, (int)'.') == NULL) { /* il doit en rester 1 */
                 trace(1, Y(t07),np,s1);
                 syslog(LOG_WARNING,Y(a07),np,s1);
                 continue;
           }
           Elt = ajouteElt(s1,-1);
           if (Elt->stat == -1) Elt->stat = isDeny(s1); /* nouveau */
           addElt(np,Elt);
         }
      }
      if (REQ) {
         printf(Y(m32));
         if ((n2 = getline(&cmd, &lc, stdin)) > 0) {
            switch (*cmd) {
               case 'C' :
                  REQ = 0;
                  break;
               case 'S' :
                  RUN = 0;
                  REQ = 0;
                  break;
               default :
                  printf(Y(m33));
                  printf(Y(m34));
                  break;
            }
         }
      }
   }
   if (REINI==0) {
      if (ENDT==1) syslog(LOG_INFO,Y(m35));
      else syslog(LOG_WARNING,Y(m36),ENDT-1);
   }
   free(line);
   free(cmd);
   kill(pid,SIGTERM);
   fclose(fp);
   close(p1[0]);
   close(p1[1]);
   closelog();
   if (RUN && (ENDT||REINI)) { /* relance auto */
       trace(1, Y(t08),strPID, DELAYR);
       sleep(DELAYR); /* attend N s */
       NP = (char**)malloc((sizeof(Pars))*(N+3));
       ie=0;
       for (i=0;i<N;i++) { NP[i] = P[i]; if (strcmp(P[i],strR) == 0) ie=1; }
       if (ie == 0) {
          NP[i++]=strR; 
          NP[i++]=strPID; 
       }
       NP[i]=NULL; 
       if (WH) write_history(FHISTO);
       comsh(Y(c21),0);
       execv(P[0],NP);
       perror(Y(p11));
   }
   pthread_join(thid,NULL);
   printf(Y(m37));
   return 0;
}


